@@ -121,14 +121,16 @@ core.
 
 use 5.010;
 use strict;
-use parent                      qw( Perl::Dist::WiX 
-                                    Perl::Dist::Strawberry::Libraries );
-use File::Spec::Functions       qw( catfile catdir  );
-use URI::file                   qw();
-use File::ShareDir              qw();
-require Perl::Dist::WiX::Util::Machine;
+use parent                           qw( Perl::Dist::WiX 
+                                         Perl::Dist::Strawberry::Libraries );
+use File::Spec::Functions            qw( catfile catdir  );
+use URI::file                        qw();
+use File::ShareDir                   qw();
+use Perl::Dist::WiX::Util::Machine   qw();
+use File::List::Object               qw();
+use Path::Class::Dir                 qw();
 
-our $VERSION = '2.02';
+our $VERSION = '2.10';
 $VERSION =~ s/_//ms;
 
 #####################################################################
@@ -138,7 +140,7 @@ $VERSION =~ s/_//ms;
 
 =head2 default_machine
 
-  Perl::Dist::Strawberry->default_machine->run;
+  Perl::Dist::Strawberry->default_machine(...)->run();
   
 The C<default_machine> class method is used to setup the most common
 machine for building Strawberry Perl.
@@ -157,7 +159,6 @@ sub default_machine {
 	# Create the machine
 	my $machine = Perl::Dist::WiX::Util::Machine->new(
 		class => $class,
-		skip  => [6],
 		@_,
 	);
 
@@ -165,25 +166,34 @@ sub default_machine {
 	$machine->add_dimension('version');
 	$machine->add_option('version',
 		perl_version => '589',
-	    build_number => 4,
+	    build_number => 5,
+	);
+	$machine->add_option('version',
+		perl_version => '589',
+	    build_number => 5,
+		image_dir    => 'D:\strawberry',
+		msi          => 1,
+		zip          => 0,
 	);
 	$machine->add_option('version',
 		perl_version => '5101',
 	);
+	$machine->add_option('version',
+		perl_version => '5101',
+		image_dir    => 'D:\strawberry',
+		msi          => 1,
+		zip          => 0,
+	);
 	$machine->add_option('version',
 		perl_version => '5101',
 		portable     => 1,
 	);
-
-	# Set the different paths
-	$machine->add_dimension('drive');
-	$machine->add_option('drive',
-		image_dir => 'C:\strawberry',
-	);
-	$machine->add_option('drive',
-		image_dir => 'D:\strawberry',
-		msi       => 1,
-		zip       => 0,
+	$machine->add_option('version',
+		perl_version => '5120',
+		relocatable  => 1,
+		gcc_version  => 4,
+		build_number => 0,
+		download_dir => 'C:\tmp\dl-gcc4',
 	);
 
 	return $machine;
@@ -198,10 +208,10 @@ sub default_machine {
 
 # Apply default paths
 sub new {
-	my $dist_dir = File::ShareDir::dist_dir('Perl-Dist-Strawberry');
+	my $dist_dir = Path::Class::Dir->new(File::ShareDir::dist_dir('Perl-Dist-Strawberry'));
 	my $class = shift;
 	
-	if ($Perl::Dist::WiX::VERSION < '1.102') {
+	if ($Perl::Dist::WiX::VERSION < '1.102102') {
 		PDWiX->throw('Perl::Dist::WiX version is not high enough.')
 	}
 
@@ -216,14 +226,15 @@ sub new {
 		perl_version         => '5101',
 		
 		# Program version.
-		build_number         => 1,
+		build_number         => 2,
+#		beta_number          => 0,
 		
 		# New options for msi building...
-		msi_license_file     => catfile($dist_dir, 'License-short.rtf'),
+		msi_license_file     => $dist_dir->file('License-short.rtf'),
 		msi_product_icon     => catfile(File::ShareDir::dist_dir('Perl-Dist-WiX'), 'win32.ico'),
 		msi_help_url         => 'http://www.strawberryperl.com/support.html',
-		msi_banner_top       => catfile($dist_dir, 'StrawberryBanner.bmp'),
-		msi_banner_side      => catfile($dist_dir, 'StrawberryDialog.bmp'),
+		msi_banner_top       => $dist_dir->file('StrawberryBanner.bmp'),
+		msi_banner_side      => $dist_dir->file('StrawberryDialog.bmp'),
 
 		# Set e-mail to something Strawberry-specific.
 		perl_config_cf_email => 'win32-vanilla@perl.org',
@@ -245,8 +256,12 @@ sub new {
 			'install_strawberry_modules_2',
 			'install_strawberry_modules_3',
 			'install_strawberry_modules_4',
+			'install_strawberry_modules_5',
+			'install_strawberry_files',
 			'add_forgotten_files',
+			'install_relocatable',
 			'regenerate_fragments',
+			'find_relocatable_fields',
 			'write_merge_module',
 			'install_win32_extras',
 			'install_strawberry_extras',
@@ -279,15 +294,7 @@ sub app_ver_name {
 
 sub add_forgotten_files {
 	my $self = shift;
-	
-	$self->add_to_fragment('IO_Scalar', 
-		[ catfile($self->image_dir(), qw( perl site lib auto IO Stringy .packlist )) ]
-	);
 
-	$self->add_to_fragment('Digest_HMAC_MD5', 
-		[ catfile($self->image_dir(), qw( perl site lib auto Digest HMAC .packlist )) ]
-	);
-	
 	return 1;
 }
 
@@ -298,9 +305,10 @@ sub output_base_filename {
 	'strawberry-perl'
 		. '-' . $_[0]->perl_version_human()
 		. '.' . $_[0]->build_number()
-		. ($_[0]->image_dir() =~ /^d:/i ? '-ddrive' : '')
-		. ($_[0]->portable() ? '-portable' : '')
-		. ($_[0]->beta_number() ? '-beta-' . $_[0]->beta_number() : '')
+		. ($_[0]->image_dir() =~ /^d:/i ? '-ddrive' : q{})
+		. ($_[0]->portable() ? '-portable' : q{})
+		. (( 64 == $_[0]->bits() ) ? q{-64bit} : q{})
+		. ($_[0]->beta_number() ? '-beta-' . $_[0]->beta_number() : q{})
 }
 
 
@@ -322,31 +330,47 @@ sub install_strawberry_c_libraries {
 	my $self = shift;
 
 	# XML Libraries
-	$self->install_zlib();
-	$self->install_libiconv();
-	$self->install_libxml();
-	$self->install_expat();
-	$self->install_libxslt();
+	$self->install_librarypacks(qw{
+		zlib
+		libiconv
+		libxml2
+		libexpat
+		libxslt
+	});
 
 	# Math Libraries
-	$self->install_gmp();
+	$self->install_librarypacks(qw{
+		gmp
+		mpc
+		mpfr
+	});
 
 	# Graphics libraries
-	$self->install_libjpeg();
-	$self->install_libgif();
-	$self->install_libtiff();
-	$self->install_libpng();
-	$self->install_libgd();
-	$self->install_libfreetype();
+	$self->install_librarypacks(qw{
+		libjpeg
+		libgif
+		libtiff
+		libpng
+		libgd
+		libfreetype
+		libxpm
+		freeglut
+	});	
 	
 	# Database Libraries
-	$self->install_libdb();
-	$self->install_libgdbm();
-	$self->install_libpostgresql();
+	$self->install_librarypacks(qw{
+		libdb
+		libgdbm
+		libpostgresql
+		libmysql
+	});
+
+	# Extra compression libraries
+	$self->install_librarypack('libxz');
 
 	# Crypto libraries
-	$self->install_libopenssl();
-	
+	$self->install_librarypack('libopenssl');
+
 	return 1;
 }
 
@@ -391,12 +415,17 @@ sub install_perl_bin {
 	my $self   = shift;
 	my %params = @_;
 	my $patch  = delete($params{patch}) || [];
+	
+	# Patch this file so GDBM_File will build.
+	my @files_to_patch = ('win32/FindExt.pm');
+
+	# If we aren't a git checkout or a 5.12 version, patch up GDBM_File.
+	if ( $self->perl_version() !~ m/ \A512 | \Agit\z /msx) {
+		push @files_to_patch, qw(ext/GDBM_File/GDBM_File.xs ext/GDBM_File/GDBM_File.pm);
+	}
+
 	return $self->SUPER::install_perl_bin(
-		patch => [ qw{
-			win32/FindExt.pm
-			ext/GDBM_File/GDBM_File.xs
-			ext/GDBM_File/GDBM_File.pm
-		}, @$patch ],
+		patch => [ @files_to_patch, @$patch ],
 		%params,
 	);
 }
@@ -424,17 +453,18 @@ sub install_strawberry_modules_1 {
 		Win32::File
 		File::Remove
 		Win32::File::Object
-		Win32::API
 		Parse::Binary
 		Win32::Exe
 		Win32::EventLog
 	} );
+	$self->install_modules('Win32::API') if not 64 == $self->bits();
 
 	# Install additional math modules
-	$self->install_pari;
+	$self->install_pari() if not 64 == $self->bits();
 	$self->install_modules( qw{
 		Math::BigInt::GMP
 	} );
+	
 	# XML Modules
 	if ($self->portable()) {
 		$self->install_distribution(
@@ -442,8 +472,8 @@ sub install_strawberry_modules_1 {
 			mod_name         => 'XML::Parser',
 			makefilepl_param => [
 				'INSTALLDIRS=site',
-				'EXPATLIBPATH=' . $self->_dir(qw{ c lib     }),
-				'EXPATINCPATH=' . $self->_dir(qw{ c include }),
+				'EXPATLIBPATH=' . $self->dir(qw{ c lib     }),
+				'EXPATINCPATH=' . $self->dir(qw{ c include }),
 			],
 		);
 	} else {
@@ -452,8 +482,8 @@ sub install_strawberry_modules_1 {
 			mod_name         => 'XML::Parser',
 			makefilepl_param => [
 				'INSTALLDIRS=vendor',
-				'EXPATLIBPATH=' . $self->_dir(qw{ c lib     }),
-				'EXPATINCPATH=' . $self->_dir(qw{ c include }),
+				'EXPATLIBPATH=' . $self->dir(qw{ c lib     }),
+				'EXPATINCPATH=' . $self->dir(qw{ c include }),
 			],
 		);
 	}
@@ -492,34 +522,20 @@ sub install_strawberry_modules_2 {
 		Test::Tester
 		Test::NoWarnings
 		Test::Deep
-		IO::Scalar
+		IO::Stringy
+		Test::Exception
 	} );
-	
-	if ($self->portable()) {
-		$self->install_distribution(
-			name             => 'RKINYON/DBM-Deep-1.0013.tar.gz',
-			mod_name         => 'DBM::Deep',
-			makefilepl_param => ['INSTALLDIRS=site'],
-			buildpl_param    => ['--installdirs', 'site'],
-			force            => 1,
-		);
-	} else {
-		$self->install_distribution(
-			name             => 'RKINYON/DBM-Deep-1.0013.tar.gz',
-			mod_name         => 'DBM::Deep',
-			makefilepl_param => ['INSTALLDIRS=vendor'],
-			buildpl_param    => ['--installdirs', 'vendor'],
-			force            => 1,
-		);
-	}
-	
+	$self->install_module(
+		name => 'DBM::Deep',
+		force => 1, # RT#56512. (missing t\lib directory) Other tests pass.
+	);
 	$self->install_modules( qw{
 		YAML::Tiny
 		PAR
 		PAR::Repository::Query
 		PAR::Repository::Client
 	} );
-	$self->install_ppm;
+	$self->install_ppm();
 
 	my $cpan_sources = catdir($self->image_dir, 'cpan', 'sources');
 	unless (-d $cpan_sources) {
@@ -562,27 +578,40 @@ sub install_strawberry_modules_3 {
 	} ) if ($self->perl_version() >= 5100);
 	
 	# Support for other databases.
+	# DB_File had a "Permission denied" on 64-bit Win7 machine on the last test. 
+	# Could be artifact of build environment... 
+	$self->install_module(
+		name  => 'DB_File',
+		force => 1,
+	); 
 	$self->install_modules( qw{
-		DB_File
 		BerkeleyDB
+		Win32::OLE
 		DBD::ODBC
+		DBD::ADO
 	} );
-	$self->install_dbd_mysql;
+
+	if ($self->portable()) {
+		$self->install_distribution(
+			name     => 'CAPTTOFU/DBD-mysql-4.014.tar.gz',
+			mod_name => 'DBD::mysql',
+			force    => 1,
+			makefilepl_param => ['INSTALLDIRS=site', '--mysql_config=mysql_config'],
+		);
+	} else {
+		$self->install_distribution(
+			name     => 'CAPTTOFU/DBD-mysql-4.014.tar.gz',
+			mod_name => 'DBD::mysql',
+			force    => 1,
+			makefilepl_param => ['INSTALLDIRS=vendor', '--mysql_config=mysql_config'],
+		);
+	}
+			
 	$self->install_module(
 		name  => 'DBD::Pg',
 		force => 1,
 	);
-
-	my $library_location = $self->library_directory();
-	my $install_location = $self->portable() ? q{perl\site\lib} : q{perl\vendor\lib};
 	
-	my $filelist = $self->install_binary(
-		name       => 'db_libraries',
-		url        => $self->_binary_url("$library_location/MySQLLibraries-20100121.zip"),
-		install_to => $install_location
-	);
-	$self->insert_fragment( 'db_libraries', $filelist );
-
 	# JSON and local library installation
 	$self->install_modules( qw{
 		common::sense
@@ -601,12 +630,13 @@ sub install_strawberry_modules_3 {
 sub install_strawberry_modules_4 {
 	my $self = shift;
 
+	# This is due to RT #52408.
+	return 1 if 64 == $self->bits();
+	
 	# Required for Net::SSLeay.
 	local $ENV{'OPENSSL_PREFIX'} = catdir($self->image_dir, 'c');
 	# This is required for IO::Socket::SSL.
 	local $ENV{'SKIP_RNG_TEST'} = 1;
-	# This is required for Net::SSH::Perl.
-	local $ENV{'HOME'} = $ENV{'USERPROFILE'};
 
 	# We have to tell the Makefile.PL where the OpenSSL 
 	# libraries are by passing a parameter for Crypt::SSLeay.
@@ -630,9 +660,9 @@ sub install_strawberry_modules_4 {
 	
 	$self->install_modules( qw{
 		Net::SSLeay
-		Digest::HMAC_MD5
+		Digest::HMAC
 		IO::Socket::SSL
-		Net::SMTP::TLS	
+		Net::SMTP::TLS
 	});
 
 	# The rest of the Net::SSH::Perl toolchain.
@@ -682,7 +712,6 @@ sub install_strawberry_modules_4 {
 		Crypt::CAST5_PP
 		Crypt::RIPEMD160
 		Crypt::Twofish
-		Test::Exception
 		Crypt::OpenPGP
 		Algorithm::Diff
 		Text::Diff
@@ -692,37 +721,117 @@ sub install_strawberry_modules_4 {
 	return 1;
 }
 
+sub install_strawberry_modules_5 {
+	my $self = shift;
 
+	# These are common requests.
+	$self->install_modules( qw{
+		File::Slurp
+		Task::Weaken
+		SOAP::Lite
+	});
+	
+	# For the local-lib script.
+	$self->install_modules( qw{
+		IO::Interactive
+		App::local::lib::Win32Helper
+	});
+
+	# Additional compression modules
+	$self->install_module( name => 'Compress::Raw::Lzma' );
+	$self->install_module( name => 'IO::Compress::Lzma', force => 1 );
+	
+	# Additional math modules.
+	$self->install_modules( qw{
+		Math::MPFR
+		Math::MPC
+	});
+	
+	# Clear things out.
+	$self->remake_path($self->dir(qw(cpan build))); 
+
+	return 1;
+}
+
+sub install_strawberry_files {
+	my $self = shift;
+	
+	## Now let's copy individual files in.
+	
+	# Copy the module-version script in, and use the runperl.bat trick on it.
+	$self->copy_file(
+		catfile($self->dist_dir(), 'module-version'), 
+		$self->file(qw(perl bin module-version))
+	);
+	$self->copy_file(
+		$self->file(qw(perl bin runperl.bat)), 
+		$self->file(qw(perl bin module-version.bat))
+	);
+	
+	# Make sure it gets installed.
+	$self->insert_fragment('module_version',
+		File::List::Object->new()->add_files(
+			$self->file(qw(perl bin module-version)),
+			$self->file(qw(perl bin module-version.bat)),				
+		),
+	);
+
+	if ($self->relocatable()) {
+		# Copy the relocation information in.
+		$self->patch_file('strawberry-merge-module.reloc.txt', $self->image_dir());
+		
+		# Make sure it gets installed.
+		$self->insert_fragment('relocation_info',
+			File::List::Object->new()->add_file(
+				$self->file('strawberry-merge-module.reloc.txt'),				
+			),
+		);
+	}
+
+	return 1;
+}
 
 
 
 #####################################################################
 # Customisations to Windows assets
 
+sub _dist_file {
+	return File::ShareDir::dist_file('Perl-Dist-Strawberry', @_);
+}
+
 sub install_strawberry_extras {
 	my $self = shift;
 
 	my $dist_dir = File::ShareDir::dist_dir('Perl-Dist-Strawberry');
-	
+
 	# Links to the Strawberry Perl website.
 	# Don't include this for non-Strawberry sub-classes
 	if ( ref($self) eq 'Perl::Dist::Strawberry' ) {
 		if (not $self->portable()) {
+			$self->install_launcher(
+				name => 'Check installed versions of modules',
+				bin  => 'module-version',
+			);
+			$self->install_launcher(
+				name => 'Create local library areas',
+				bin  => 'llw32helper',
+			);
 			$self->install_website(
 				name       => 'Strawberry Perl Website',
 				url        => $self->strawberry_url(),
-				icon_file  => catfile($dist_dir, 'strawberry.ico')
+				icon_file  => _dist_file('strawberry.ico')
 			);
 			$self->install_website(
 				name       => 'Strawberry Perl Release Notes',
 				url        => $self->strawberry_release_notes_url(),
-				icon_file  => catfile($dist_dir, 'strawberry.ico')
+				icon_file  => _dist_file('strawberry.ico')
 			);
 			# Link to IRC.
 			$self->install_website(
 				name       => 'Live Support',
 				url        => 'http://widget.mibbit.com/?server=irc.perl.org&channel=%23win32',
-				icon_file  => catfile($dist_dir, 'onion.ico')
+				icon_file  => _dist_file('onion.ico')
 			);
 		}
 		$self->patch_file( 'README.txt' => $self->image_dir(), { dist => $self } );
@@ -730,14 +839,30 @@ sub install_strawberry_extras {
 
 	my $license_file_from = catfile($dist_dir, 'License.rtf');
 	my $license_file_to = catfile($self->license_dir(), 'License.rtf');
-	my $readme_file = catfile($self->image_dir(), 'README.txt');
+	my $readme_file = $self->file('README.txt');
 
-	$self->_copy($license_file_from, $license_file_to);	
+	my $onion_ico_file = $self->file(qw(win32 onion.ico));
+	my $strawberry_ico_file = $self->file(qw(win32 strawberry.ico));
+	
+	$self->copy_file($license_file_from, $license_file_to);	
 	if (not $self->portable()) {
 		$self->add_to_fragment( 'Win32Extras',
-			[ $license_file_to, $readme_file ] );
+			[ $license_file_to, $readme_file, $onion_ico_file, $strawberry_ico_file ] );
 	}
 
+	if ($self->relocatable()) {
+		# Copy the relocation information in.
+		$self->patch_file('strawberry-ui.reloc.txt', $self->image_dir());
+		
+		# Make sure it gets installed.
+		$self->insert_fragment('relocation_ui_info',
+			File::List::Object->new()->add_file(
+				$self->file('strawberry-ui.reloc.txt'),				
+			),
+		);
+	}
+
+	
 	return 1;
 }
 
@@ -747,7 +872,7 @@ sub strawberry_url {
 
 	# Strip off anything post-version
 	unless ( $path =~ s/^(strawberry-perl-\d+(?:\.\d+)+).*$/$1/ ) {
-		die "Failed to generate the strawberry subpath";
+		PDWiX->throw("Failed to generate the strawberry subpath");
 	}
 
 	return "http://strawberryperl.com/$path";
@@ -757,11 +882,58 @@ sub strawberry_release_notes_url {
 	my $self = shift;
 	my $path = $self->perl_version_human()
 		. q{.} . $self->build_number()
-		. ($self->beta_number() ? '.beta' : '');
+		. ($self->beta_number() ? '.beta-' . $self->beta_number() : '');
 
 	return "http://strawberryperl.com/release-notes/$path.html";
 }
 
+sub msi_relocation_commandline_files {
+	my $self = shift;
+	
+	return('relocation_ui_info', $self->file('strawberry-ui.reloc.txt'));
+}
+
+sub msm_relocation_commandline_files {
+	my $self = shift;
+	
+	return('relocation_info', $self->file('strawberry-merge-module.reloc.txt'));
+}
+
+sub msi_relocation_idlist {
+	my $self = shift;
+
+	my $answer;
+	my %files = $self->msi_relocation_commandline_files();
+
+	my ( $fragment, $file, $id );
+	while ( ( $fragment, $file ) = each %files ) {
+		$id = $self->get_fragment_object($fragment)->find_file_id($file);
+		PDWiX->throw("Could not find file $file in fragment $fragment\n")
+		  if not defined $id;
+		$answer .= "[#$id]";
+	}
+
+	return $answer;
+} ## end sub msi_relocation_commandline
+
+sub msm_relocation_idlist {
+	my $self = shift;
+
+	my $answer;
+	my %files = $self->msm_relocation_commandline_files();
+
+	my ( $fragment, $file, $id );
+	while ( ( $fragment, $file ) = each %files ) {
+		$id = $self->get_fragment_object($fragment)->find_file_id($file);
+		PDWiX->throw("Could not find file $file in fragment $fragment\n")
+		  if not defined $id;
+		$answer .= "[#$id]";
+	}
+
+	return $answer;
+} ## end sub msi_relocation_commandline
+
+
 1;
 
 =pod
